@@ -2,7 +2,7 @@
 import logging
 import numpy as np
 from numpy.polynomial.polynomial import Polynomial
-from scipy.ndimage import median_filter
+from silx.math.medianfilter import medfilt2d
 import scipy.fft  # pylint: disable=E0611
 from ..utils import previouspow2
 from ..misc import fourier_filters
@@ -38,6 +38,7 @@ local_ifftn = scipy.fft.irfftn
 
 class AlignmentBase:
     default_extra_options = {"blocking_plots": False}
+    _default_cor_options = {}
 
     def __init__(
         self,
@@ -47,6 +48,7 @@ class AlignmentBase:
         logger=None,
         data_type=np.float32,
         extra_options=None,
+        cor_options=None,
     ):
         """
         Alignment basic functions.
@@ -71,6 +73,13 @@ class AlignmentBase:
 
         self._init_parameters(vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=extra_options)
         self._plot_windows = {}
+        self._set_cor_options(cor_options)
+
+    def _set_cor_options(self, cor_options):
+        default_dict = self._default_cor_options.copy()
+        if cor_options is not None:
+            default_dict.update(cor_options)
+        self.cor_options = default_dict
 
     def _init_parameters(self, vert_fft_width, horz_fft_width, verbose, logger, data_type, extra_options=None):
         self.logger = LoggerOrPrint(logger)
@@ -273,7 +282,7 @@ class AlignmentBase:
                     vertex_x, vertex_min_x, vertex_max_x
                 )
             else:
-                message = "Fitted positions outide the input margins [{}, {}]: {} below and {} above".format(
+                message = "Fitted positions outside the input margins [{}, {}]: {} below and {} above".format(
                     vertex_min_x,
                     vertex_max_x,
                     np.sum(1 - lower_bound_ok),
@@ -463,7 +472,7 @@ class AlignmentBase:
                     median_filt_shape,
                 )
             )
-            img = median_filter(img, size=median_filt_shape)
+            img = medfilt2d(img, kernel_size=median_filt_shape)
         return img
 
     def _transform_to_fft(
