@@ -548,7 +548,6 @@ class ChunkedPipeline:
 
         if options["method"] == "FBP":
             n_slices = self.n_slices
-            radios_shape_for_sino_builder = self.radios_cropped_shape
             self.reconstruction = self.FBPClass(
                 self.sinos_shape[1:],
                 angles=options["angles"],
@@ -567,7 +566,6 @@ class ChunkedPipeline:
             )
 
         if options["method"] == "cone":
-            radios_shape_for_sino_builder = self.radios_shape
             n_slices = self.n_slices + sum(self.margin[0])
             # For numerical stability, normalize all lengths with respect to detector pixel size
             pixel_size_m = self.dataset_info.pixel_size * 1e-6
@@ -577,17 +575,18 @@ class ChunkedPipeline:
                 (self.radios_shape[1],) + self.sino_shape,
                 source_sample_dist,
                 sample_detector_dist,
-                angles=options["angles"],
-                # TODO one center for each angle to handle "x translations"
+                angles=-options["angles"],
                 rot_center=options["rotation_axis_position"],
+                axis_correction=(-options["axis_correction"] if options["axis_correction"] is not None else None),
                 pixel_size=1,
+                scale_factor=1.0 / options["voxel_size_cm"][0],
+                padding_mode=options["padding_type"],
+                slice_roi=self.process_config.rec_roi,
             )
 
         self._allocate_recs(*self.process_config.rec_shape, n_slices=n_slices)
         n_a, _, n_x = self.radios_cropped_shape
         self._tmp_sino = self._allocate_array((n_a, n_x), "f", name="tmp_sino")
-        if options["method"] == "cone":
-            self.sinos = self._allocate_array(self.sino_builder.output_shape, "f", name="sinos")
 
     @use_options("histogram", "histogram")
     def _init_histogram(self):
@@ -734,14 +733,25 @@ class ChunkedPipeline:
         """
         This reconstructs the entire sinograms stack at once
         """
-        self.sino_builder.get_sinos(self.radios, output=self.sinos)
+
+        n_angles, n_z, n_x = self.radios.shape
+
+        # FIXME
+        # can't do a discontiguous single copy...
+        sinos_contig = self._allocate_array((n_z, n_angles, n_x), np.float32, "sinos_cone")
+        for i in range(n_z):
+            sinos_contig[i] = self.radios[:, i, :]
+        # ---
+
+        # In principle radios are not cropped at this stage,
+        # so self.sub_region[2][0] can be used instead of self.get_slice_start_index() instead of self.sub_region[2][0]
         z_min, z_max = self.sub_region_xz[2:]
-        n_z = self.process_config.radio_shape(binning=True)[0]
+        n_z_tot = self.process_config.radio_shape(binning=True)[0]
 
         self.reconstruction.reconstruct(  # pylint: disable=E1101
-            self.sinos,
+            sinos_contig,
             output=self.recs,
-            relative_z_position=((z_min + z_max) / self.process_config.binning_z / 2) - n_z / 2,
+            relative_z_position=((z_min + z_max) / self.process_config.binning_z / 2) - n_z_tot / 2,
         )
 
     @pipeline_step("histogram", "Computing histogram")
