@@ -128,7 +128,13 @@ def estimate_required_memory(
         if process_config.rec_params["method"] == "cone":
             # In cone-beam reconstruction, need both sinograms and reconstruction inside GPU.
             # That's big!
-            total_memory_needed += 2 * data_volume_size
+            # Even more if padding is done at the nabu side.
+            if rec_config["padding_type"] == "zeros":
+                total_memory_needed += 2 * data_volume_size
+            else:
+                total_memory_needed += (
+                    data_volume_size + get_next_power(2 * Nx) * Na * Nz * 4 * 4
+                )  # no idea why the last *4 is necessary
 
     if debug:
         print(
