@@ -29,7 +29,7 @@ class FullFieldDatasetValidator(DatasetValidatorBase):
     def _check_slice_indices(self):
         nx, nz = self.dataset_info.radio_dims
         rec_params = self.rec_params
-        if rec_params["enable_halftomo"]:
+        if self.is_halftomo:
             ny, nx = self._get_nx_ny()
         what = (("start_x", "end_x", nx), ("start_y", "end_y", nx), ("start_z", "end_z", nz))
         for start_name, end_name, numels in what:
@@ -61,3 +61,7 @@ class FullFieldDatasetValidator(DatasetValidatorBase):
                 self.logger.warning(
                     "Cone-beam reconstruction: 'sample_detector_dist' not provided, will use the one in dataset metadata"
                 )
+            if self.is_halftomo:
+                err_msg = "Cone-beam reconstruction with half-acquisition is not supported yet"
+                self.logger.fatal(err_msg)
+                raise NotImplementedError(err_msg)
