@@ -517,10 +517,15 @@ class ProcessConfig(ProcessConfigBase):
         #
         # Translation movements
         #
+
         translations = dataset_info.translations
         if translations is not None:
-            tasks.append("radios_movements")
-            options["radios_movements"] = {"translation_movements": dataset_info.translations}
+            if np.max(np.abs(translations[:, 1])) < 1e-5:
+                self.logger.warning("No vertical translation greater than 1e-5 - disabling vertical shifts")
+                # horizontal movements are handled in backprojector
+            else:
+                tasks.append("radios_movements")
+                options["radios_movements"] = {"translation_movements": dataset_info.translations}
         #
         # Sinogram normalization (before half-tomo)
         #
@@ -600,7 +605,7 @@ class ProcessConfig(ProcessConfigBase):
             )
             if all([m is not None for m in mean_positions_xyz]):
                 rec_options["position"] = mean_positions_xyz
-            if rec_options["sample_detector_dist"] is None:
+            if rec_options["method"] == "cone" and rec_options["sample_detector_dist"] is None:
                 rec_options["sample_detector_dist"] = self.dataset_info.distance  # was checked to be not None earlier
 
             # New key
