@@ -359,6 +359,16 @@ class FullFieldReconstructor:
 
         return (max_overlap, 0)
 
+    def _compute_rotation_margin(self):
+        if "rotate_projections" in self.process_config.processing_steps:
+            # Partial radios rotation yields too much error in single-slice mode
+            # Forcing a big margin circumvents the problem
+            # This is likely to trigger the 'grouped mode', but perhaps grouped mode should always be used when rotating radios
+            nz, nx = self.process_config.radio_shape(binning=True)
+            return nz // 3, nx // 3
+        else:
+            return 0, 0
+
     def _ensure_good_chunk_size_and_margin(self):
         """
         Check that "chunk_size" and "margin" (if any) are a multiple of binning factor.
