@@ -1,5 +1,5 @@
 from nabu.pipeline.helical import gridded_accumulator, span_strategy
-from nabu.testutils import get_data
+from nabu.testutils import get_data, __do_long_tests__
 
 import pytest
 import numpy as np
@@ -49,6 +49,7 @@ def bootstrap(request):
     cls.rtol_regridded = 1.0e-6
 
 
+@pytest.mark.skipif(not (__do_long_tests__), reason="need environment variable NABU_LONG_TESTS=1")
 @pytest.mark.usefixtures("bootstrap")
 class TestGriddedAccumulator:
     """
