@@ -1,12 +1,15 @@
 import numpy as np
-from pycuda import gpuarray as garray
-from ..utils import calc_padding_lengths, updiv, get_cuda_srcfile
+from ..utils import calc_padding_lengths, updiv, get_cuda_srcfile, docstring
 from ..cuda.processing import CudaProcessing
-from ..cuda.kernel import CudaKernel
-from ..cuda.padding import CudaPadding
+from ..cuda.utils import __has_pycuda__
+from ..processing.padding_cuda import CudaPadding
+from ..processing.fft_cuda import get_fft_class
 from .phase_cuda import CudaPaganinPhaseRetrieval
 from .ctf import CTFPhaseRetrieval
 
+if __has_pycuda__:
+    from pycuda import gpuarray as garray
+
 
 # TODO:
 #  - better padding scheme (for now 2*shape)
@@ -17,6 +20,7 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
     Cuda back-end of CTFPhaseRetrieval
     """
 
+    @docstring(CTFPhaseRetrieval)
     def __init__(
         self,
         shape,
@@ -29,9 +33,11 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
         lim1=1.0e-5,
         lim2=0.2,
         use_rfft=True,
-        fftw_num_threads=None,
+        fftw_num_threads=None,  # COMPAT.
+        fft_num_threads=None,
         logger=None,
         cuda_options=None,
+        fft_backend="skcuda",
     ):
         """
         Initialize a CudaCTFPhaseRetrieval.
@@ -62,30 +68,26 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
             lim2=lim2,
             logger=logger,
             use_rfft=True,
-            fftw_num_threads=False,
+            fft_num_threads=False,
         )
         self._init_ctf_filter()
         self._init_cuda_padding()
-        self._init_fft()
+        self._init_fft(fft_backend)
         self._init_mult_kernel()
 
     def _init_ctf_filter(self):
         self._mean_scale_factor = self.unreg_filter_denom[0, 0] * np.prod(self.shape_padded)
-        self._d_filter_num = garray.to_gpu(self.unreg_filter_denom).astype("f")
-        self._d_filter_denom = garray.to_gpu(
-            (1.0 / (2 * self.unreg_filter_denom * self.unreg_filter_denom + self.lim)).astype("f")
+        self._d_filter_num = self.cuda_processing.to_device("_d_filter_num", self.unreg_filter_denom).astype("f")
+        self._d_filter_denom = self.cuda_processing.to_device(
+            "_d_filter_denom", (1.0 / (2 * self.unreg_filter_denom * self.unreg_filter_denom + self.lim)).astype("f")
         )
 
     def _init_cuda_padding(self):
         pad_width = calc_padding_lengths(self.shape, self.shape_padded)
         # Custom coordinate transform to get directly FFT layout
-        R, C = np.indices(self.shape, dtype=np.int32)
-        coords_R = np.roll(
-            np.pad(R, pad_width, mode=self.padding_mode), (-pad_width[0][0], -pad_width[1][0]), axis=(0, 1)
-        )
-        coords_C = np.roll(
-            np.pad(C, pad_width, mode=self.padding_mode), (-pad_width[0][0], -pad_width[1][0]), axis=(0, 1)
-        )
+        R, C = np.indices(self.shape, dtype=np.int32, sparse=True)
+        coords_R = np.roll(np.pad(R.ravel(), pad_width[0], mode=self.padding_mode), -pad_width[0][0])
+        coords_C = np.roll(np.pad(C.ravel(), pad_width[1], mode=self.padding_mode), -pad_width[1][0])
         self.cuda_padding = CudaPadding(
             self.shape,
             (coords_R, coords_C),
@@ -93,16 +95,14 @@ class CudaCTFPhaseRetrieval(CTFPhaseRetrieval):
             # propagate cuda options ?
         )
 
-    def _init_fft(self):
-        # Import has to be done here, otherwise scikit-cuda creates a cuda/cublas context at import
-        from silx.math.fft.cufft import CUFFT
-
-        self.cufft = CUFFT(template=np.zeros(self.shape_padded, dtype="f"))
-        self.d_radio_padded = self.cufft.data_in
-        self.d_radio_f = self.cufft.data_out
+    def _init_fft(self, fft_backend):
+        fft_cls = get_fft_class(backend=fft_backend)
+        self.cufft = fft_cls(shape=self.shape_padded, dtype=np.float32, r2c=True)
+        self.d_radio_padded = self.cuda_processing.allocate_array("d_radio_padded", self.shape_padded, "f")
+        self.d_radio_f = self.cuda_processing.allocate_array("d_radio_f", self.cufft.shape_out, np.complex64)
 
     def _init_mult_kernel(self):
-        self.cpxmult_kernel = CudaKernel(
+        self.cpxmult_kernel = self.cuda_processing.kernel(
             "CTF_kernel",
             filename=get_cuda_srcfile("ElementOp.cu"),
             signature="PPPfii",
