@@ -1,7 +1,7 @@
 import numpy as np
 import pytest
 from nabu.reconstruction.rings_cuda import CudaSinoMeanDeringer
-from nabu.testutils import compare_arrays, get_data, generate_tests_scenarios, __do_long_tests__
+from nabu.testutils import compare_arrays, get_data, generate_tests_scenarios, __do_long_tests__, __do_large_mem_tests__
 from nabu.reconstruction.rings import MunchDeringer, SinoMeanDeringer, VoDeringer, __has_algotom__
 from nabu.thirdparty.pore3d_deringer_munch import munchetal_filter
 from nabu.cuda.utils import __has_pycuda__, get_cuda_context
@@ -144,8 +144,8 @@ class TestMunchDeringer:
         # TODO check result. The generated test sinogram is "too synthetic" for this kind of deringer
 
     @pytest.mark.skipif(
-        not (__have_tomocupy_deringer__),
-        reason="Need cupy for this test",
+        not (__have_tomocupy_deringer__ and __do_large_mem_tests__),
+        reason="Need cupy for this test, and use NABU_LARGE_MEM_TESTS",
     )
     def test_cuda_vo_deringer(self):
         # Beware, this deringer seems to be buggy for "too-small" sinograms
