@@ -3,10 +3,11 @@ import pytest
 from nabu.testutils import get_data, generate_tests_scenarios, compare_shifted_images
 from nabu.cuda.utils import get_cuda_context, __has_pycuda__, __has_cufft__
 from nabu.opencl.utils import get_opencl_context, __has_pyopencl__
+from nabu.processing.fft_opencl import has_vkfft as has_vkfft_cl
 from nabu.thirdparty.algotom_convert_sino import extend_sinogram
 
 __has_pycuda__ = __has_pycuda__ and __has_cufft__  # need both for using Cuda backprojector
-
+__has_pyopencl__ = __has_pyopencl__ and has_vkfft_cl()
 
 if __has_pycuda__:
     from nabu.reconstruction.fbp import CudaBackprojector
