@@ -301,7 +301,7 @@ def check_overlaps(frames: Union[tuple, numpy.ndarray], positions: tuple, axis:
         if raise_error:
             raise ValueError(error_msg)
         else:
-            _logger.error(raise_error)
+            _logger.error(error_msg)
 
     # convert each frame to appropriate bounding box according to the axis
     def convert_to_bb(frame: numpy.ndarray, position: tuple, axis: int):
