@@ -695,11 +695,6 @@ def test_DistributePostProcessZStitcher(tmp_path, configuration_dist, flip_ud):
     if complete:
         concatenate_volumes(output_volume=final_vol, volumes=tuple(reconstructed_sub_volumes), axis=1)
         final_vol.load_data(store=True)
-        with h5py.File("debug.hdf5", mode="w") as h5f:
-            h5f["raw_volume"] = raw_volume
-            h5f["final_vol"] = final_vol.data
-            h5f["volume1"] = volume_1.data
-            h5f["volume2"] = volume_2.data
         numpy.testing.assert_almost_equal(
             raw_volume,
             final_vol.data,
@@ -925,14 +920,6 @@ def test_vol_z_stitching_with_alignment_axis_2(tmp_path, alignment_axis_2):
 
     assert output_volume.data.shape == (120, 4, 120)
 
-    import h5py
-
-    with h5py.File("input.h5", mode="w") as h5f:
-        h5f["data"] = raw_volume
-
-    with h5py.File("output.h5", mode="w") as h5f:
-        h5f["data"] = output_volume.data
-
     if alignment_axis_2 == "center":
         numpy.testing.assert_array_almost_equal(raw_volume[:, :, 10:-10], output_volume.data[:, :, 10:-10])
     elif alignment_axis_2 == "left":
